/*
  Copyright (C) 2013 Harry

 This file is part of spacewar.

 spacewar is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 spacewar is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with spacewar.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "ships.h"

// Initializes the number of ships passed as argument.
void initialize_ships(int number){
  if(number < NUMBER_OF_SHIPS){
    int i;
    for(i = NUMBER_OF_SHIPS - 1; i > number - 1; i --)
      ship[i].body =  NULL;
  }
  if(number < 1)
    return;
  // The first ship have 3 polygons:
  ship[0].body = (polygon **) malloc(4*sizeof(polygon *));
  ship[0].body[0] = new_polygon(4, 
				130.0, 159.0,
				125.0, 164.0, // The fuel tank
				119.0, 159.0,
				125.0, 153.0);
  ship[0].body[1] = new_polygon(7,
				133.0, 145.0,
				150.0, 184.0,
				111.0, 167.0,
				122.0, 162.0, // The ship body
				125.0, 164.0,
				130.0, 159.0,
				128.0, 156.0);
  ship[0].body[2] = new_polygon(2,
				144.0, 173.0, // The cockpit
				139.0, 178.0);
  ship[0].body[3] = NULL; // The array must end with a NULL pointer. 
  ship[0].propulsor[0] = ship[0].body[0];
  ship[0].propulsor[1] = NULL; // This ship has only 1 propulsor
  ship[0].front = ship[0].body[1] -> next;
  ship[0].color = BLUE;
  ship[0].x = 136.0; ship[0].y = 170.0;
  ship[0].dx = ship[0].dy = 0.0;
  ship[0].status = ALIVE;
  gettimeofday(&(ship[0].time), NULL);
  ship[0].fuel = FUEL;
  if(number < 2)
    return;
  // The second ship have 7 polygons
  ship[1].body = (polygon **) malloc(8*sizeof(polygon *));
  ship[1].body[0] = new_polygon(2,
				650.0, 614.0, // A ship division
				657.0, 620.0);
  ship[1].body[1] = new_polygon(7,
				649.0, 628.0,
				664.0, 613.0,
				665.0, 616.0,
				666.0, 626.0, // The ship body
				675.0, 638.0,
				662.0, 630.0,
				652.0, 629.0);
  ship[1].body[2] = new_polygon(3,
				649.0, 628.0,
				650.0, 614.0, // The cockpit
				664.0, 613.0);
  ship[1].body[3] = new_polygon(2,
				668.0, 636.0, // Left wing
				668.0, 645.0);
  ship[1].body[4] = new_polygon(2,
				673.0, 632.0, // Right wing
				681.0, 632.0);
  ship[1].body[5] = new_polygon(2,
				674.0, 625.0, // Right propulsor
				691.0, 642.0);
  ship[1].body[6] = new_polygon(2,
				662.0, 638.0,  // Left propulsor
				678.0, 654.0);
  ship[1].body[7] = NULL; // The NULL pointer marks the end of array
  ship[1].propulsor[0] = ship[1].body[5];
  ship[1].propulsor[1] = ship[1].body[6];
  ship[1].front = ship[1].body[2] -> next;
  ship[1].color = RED;
  ship[1].x = 665.0; ship[1].y = 630.0;
  ship[1].dx = ship[1].dy = 0.0;
  ship[1].status = ALIVE;
  gettimeofday(&(ship[1].time), NULL);
  ship[1].fuel = FUEL;
}


// Free the memory used by the initialized ships
void destroy_ships(void){
  int i;
  for(i = 0; i < NUMBER_OF_SHIPS; i ++){
    if(ship[i].body != NULL){
      int j;
      for(j = 0; ship[i].body[j] != NULL; j ++)
	destroy_polygon(ship[i].body[j]);
    }
  }
}

// Film initialized ships with a camera passed as argument
void film_ships(camera *cam){
  int p, i;
  for(i = 0; i < NUMBER_OF_SHIPS; i ++){
    if(ship[i].body != NULL && ship[i].status == ALIVE){
      for(p = 0; ship[i].body[p] != NULL; p ++){
	film_fullpolygon(cam, ship[i].body[p], BLACK);
	film_polygon(cam, ship[i].body[p], ship[i].color); 
      }
    }
  }
}

// Erase initialized ships with a camera passed as argument
void erase_ships(camera *cam){
  int p, i;
  for(i = 0; i < NUMBER_OF_SHIPS; i ++){
    if(ship[i].body != NULL && ship[i].status == ALIVE){
      for(p = 0; ship[i].body[p] != NULL; p ++){
	erase_fullpolygon(cam, ship[i].body[p]);
	erase_polygon(cam, ship[i].body[p]);
      }
    }
  }
}

